#include "Tlist.h"
#include "Tlookuplist.h"

/// Create a new lookup list.  (really not necessary)

Tlist * Tlookup_list_new () {
   Tlist * list = Tlist_new ();
   return(list);
}

/// Safely deletes all the memory it allocates.

void Tlookup_list_delete (Tlist * list) {
   Tlist_delete_all (list);
}

/// Push another lookup word onto list.

void Tlookup_list_push_back (Tlist * list, char *s, int type, int number, int gender) {
   char * t;
   Tlookup * c = NEW(Tlookup);
   c->type = type;
   c->number = number;
   c->gender = gender;
   t = (char *)malloc(sizeof(char)*strlen(s));
   strcpy(t,s);
   c->string = t;
   Tlist_push_back (list, c);
}

/// Read a file into the lookup list.

void Tlookup_list_read_file (Tlist * list, char *filename) {
   FILE *in;
   char buffer[80];
   int ch;
   int i=0;
   int word=0;
   int type, gender, number;
   int done = 0;
   
   in = fopen(filename, "r");
   if (in == NULL) {
      printf ("Cannot open \"%s\".\n", filename);
      exit(8);
   }
   while (done == 0) {
      if (i>79) {
	 printf ("This file seems to contain words longer than 80 characters long.\n"
		 "Check to make sure your input file is valid plain text.\n\n");
	 exit(8);
      }
      ch = fgetc(in);
      switch (ch) {
       case EOF:
       case '\n':
       case '\r':
       case '\t':
       case ' ':
	 if (i>0) {
	    buffer[i] = 0;
	    switch (word) {
	     case 0:
	       type = UNKNOWN;
	       if (strcmp(&buffer[0], "pronoun") == 0)    type = PRONOUN;
	       if (strcmp(&buffer[0], "possessive") == 0) type = POSSESSIVE;
	       if (strcmp(&buffer[0], "article") == 0)    type = ARTICLE;
	       if (strcmp(&buffer[0], "verb") == 0)       type = VERB;
	       if (strcmp(&buffer[0], "join") == 0)       type = JOIN;
	       break;
	     case 1:
	       gender = UNKNOWN;
	       if (strcmp(&buffer[0], "male") == 0)    gender = MALE;
	       if (strcmp(&buffer[0], "female") == 0)  gender = FEMALE;
	       if (strcmp(&buffer[0], "thing") == 0)   gender = THING;
	       if (strcmp(&buffer[0], "nuetral") == 0) gender = NUETRAL;
	       if (strcmp(&buffer[0], "none") == 0)    gender = NONE;
	       break;
	     case 2:
	       number = UNKNOWN;
	       if (strcmp(&buffer[0], "singular") == 0)  number = SINGULAR;
	       if (strcmp(&buffer[0], "plural") == 0)    number = PLURAL;
	       if (strcmp(&buffer[0], "none") == 0)      number = NONE;
	       break;
	     case 3:
	       // After the 4th word, add it to the list.
	       Tlookup_list_push_back(list, &buffer[0], type, number, gender);
	       // Add a duplicate of each pronoun type to check for case.
	       if ((type==PRONOUN) || (type==ARTICLE) || (type==JOIN) || (type==POSSESSIVE) ){
		  buffer[0] = buffer[0] - 32;
		  Tlookup_list_push_back(list, &buffer[0], type, number, gender);
	       }
	       word=-1;
	       break;
	    }
	    if (ch == EOF) done = 1;
	    word=word+1;
	    i=0;
	 }
	 break;
       default:
	 buffer[i] = ch;
	 i=i+1;
	 break;
      }
      if (ch == EOF) break;
   }
   fclose(in);
}

/// Print the lookup list in a lisp like manner.
/// This is useful for debugging.

void Tlookup_list_print_lispish (Tlist * list) {
   Tlookup * c;
   c = Tlist_head (list);
   while (c != NULL) {
      printf("(\"%s\" %d %d %d)\n", c->string, c->type, c->number, c->gender);
      c = Tlist_next_until_tail (list);
   }
}

